import 'dart:convert';

import 'package:models_weebi/utils.dart' show DateRange, DatesWeebi;
import 'package:models_weebi/src/closings/abstract/closing_contact.dart';

import 'package:models_weebi/src/closings/closing_ledger.dart';
import 'package:models_weebi/src/closings/closing_range.dart';
import 'package:models_weebi/models.dart' show FinFlow, FinFlowType;

extension FinFlowsClosingLedgerContact on List<ClosingLedgerContact> {
  List<FinFlow> contactClFinFlows(
      int herderId, DateRange dateRange, List<FinFlow> flows) {
    //print('function dateRange start ${dateRange.startDate}');
    //print('function dateRange endDate ${dateRange.endDate}');
    //print('first c startDate ${first.closingRange.startDate}');
    //print('first c endDate ${first.closingRange.endDate}');
    //print('last c startDate ${last.closingRange.startDate}');
    //print('last c endDate ${last.closingRange.endDate}');
    forEach((c) {
      if (c.contactId == herderId) {
        if ((c.closingRange.start.isAtSameMomentAs(dateRange.start) ||
                c.closingRange.start.isAfter(dateRange.start)) &&
            (c.closingRange.end.isAtSameMomentAs(dateRange.end) ||
                c.closingRange.end.isBefore(dateRange.end))) {
          flows.sell.sumClosings += c.sell;
          flows.sellCovered.sumClosings += c.sellCovered;
          flows.sellDeferred.sumClosings += c.sellDeferred;
          flows.spend.sumClosings += c.spend;
          flows.spendCovered.sumClosings += c.spendCovered;
          flows.spendDeferred.sumClosings += c.spendDeferred;
          flows.wage.sumClosings += c.wage;
        }
      }
    });
    return flows;
  }
}

class ClosingLedgerContact extends ClosingLedger
    implements ClosingContactAbstract {
  @override
  int contactId;
  num balance;
  bool get isAllZero =>
      sell == 0 &&
      sellCovered == 0 &&
      sellDeferred == 0 &&
      spend == 0 &&
      spendDeferred == 0 &&
      spendCovered == 0 &&
      wage == 0;

  ClosingLedgerContact(
    super.closingRange, {
    required this.contactId,
    // le solde à l'instant t, avec l'ancien solde, et non pas juste le solde du mois
    // balance = previous balance + solde client - solde fournisseur - solde wages
    required this.balance,
    required super.sell,
    required super.sellCovered,
    required super.sellDeferred,
    required super.spend,
    required super.spendCovered,
    required super.spendDeferred,
    required super.wage,
  });

  static final dummyFeb = ClosingLedgerContact(
    ClosingRange.dummyFeb,
    contactId: 1,
    sell: 0,
    sellCovered: 0,
    sellDeferred: 40,
    spend: 0,
    spendCovered: 0,
    spendDeferred: 20,
    wage: 0,
    balance: -20,
  );
  static final dummyMar2020 = ClosingLedgerContact(
    ClosingRange(
      date: DateTime.now(),
      start: DatesWeebi.marStart,
      end: DatesWeebi.marEnd,
    ),
    contactId: 1,
    sell: 0,
    sellCovered: 0,
    sellDeferred: 40,
    spend: 0,
    spendCovered: 0,
    spendDeferred: 20,
    wage: 0,
    balance: -20,
  );

  @override
  bool operator ==(other) => identical(closingRange, other);

  @override
  int get hashCode => closingRange.hashCode;

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'closingRange': closingRange.toMap(),
      'contactId': contactId,
      'sell': sell,
      'balance': balance,
      'sellCovered': sellCovered,
      'sellDeferred': sellDeferred,
      'spend': spend,
      'spendCovered': spendCovered,
      'spendDeferred': spendDeferred,
      'wage': wage
    };
  }

  factory ClosingLedgerContact.fromMap(Map<String, dynamic> map) {
    return ClosingLedgerContact(
      // ignore: missing_required_param
      ClosingRange.fromMap(map['closingRange']),
      contactId: map['contactId'],
      balance: map['balance'] ?? 0,
      sell: map['sell'],
      sellCovered: map['sellCovered'],
      sellDeferred: map['sellDeferred'],
      spend: map['spend'],
      spendCovered: map['spendCovered'],
      spendDeferred: map['spendDeferred'],
      wage: map['wage'],
    );
  }
}
