import 'dart:convert';

import 'package:models_weebi/extensions.dart';
import 'package:models_weebi/utils.dart';

class ClosingRange extends DateRange {
  final DateTime date;

  ClosingRange(
      {required this.date, required DateTime start, required DateTime end})
      : super(start, end);

  bool isInRange(DateTime startDateRange, DateTime endDateRange) {
    return (start.isAfter(startDateRange) ||
            start.isAtSameMomentAs(startDateRange)) &&
        (end.isBefore(endDateRange) || end.isAtSameMomentAs(endDateRange));
  }

  static final ClosingRange dummyLastMonth = ClosingRange(
    date: DateTime.now(),
    start: DateTime(
      DateTime.now().year,
      DateTime.now().month - 1,
      1,
    ),
    end: DateTime(0).previousMonthLastDay,
  );
  static final dummyFeb = ClosingRange(
    date: DatesWeebi.febEnd,
    start: DatesWeebi.febStart,
    end: DatesWeebi.febEnd,
  );

  static final dummyMar = ClosingRange(
    date: DateTime.now(),
    start: DatesWeebi.marStart,
    end: DatesWeebi.marEnd,
  );

  static final dummyNov20ToJuly21 = ClosingRange(
    date: DateTime.now(),
    start: DateTime(2020, 11, 31, 23, 59, 59),
    end: DateTime(2021, 7, 31, 23, 59, 59),
  );

  static final dummyJuly21ToOct21 = ClosingRange(
    date: DateTime.now(),
    start: DateTime(2021, 7, 31, 23, 59, 59),
    end: DateTime(2021, 10, 31, 23, 59, 59),
  );

  Map<String, dynamic> toMap() {
    return {
      'date': date.toIso8601String(),
      'startDate': start.toIso8601String(),
      'endDate': end.toIso8601String(),
    };
  }

  factory ClosingRange.fromMap(Map<String, dynamic> map) {
    return ClosingRange(
      date: DateTime.parse(map['date']),
      start: DateTime.parse(map['startDate']),
      end: DateTime.parse(map['endDate']),
    );
  }

  String toJson() => json.encode(toMap());

  factory ClosingRange.fromJson(String source) =>
      ClosingRange.fromMap(json.decode(source));

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ClosingRange &&
        other.date == date &&
        other.start == start &&
        other.end == end;
  }

  @override
  int get hashCode {
    return date.hashCode ^ start.hashCode ^ end.hashCode;
  }

  ClosingRange copyWith({
    DateTime? date,
    DateTime? start,
    DateTime? end,
  }) {
    return ClosingRange(
      date: date ?? this.date,
      start: start ?? this.start,
      end: end ?? this.end,
    );
  }
}
