import 'dart:convert';
import 'package:models_weebi/src/closings/abstract/closing_range.dart';
import 'package:models_weebi/src/closings/closing_stock_article.dart';
import 'package:models_weebi/src/closings/closing_range.dart';
import 'package:models_weebi/src/closings/closing_stock_article_calibre.dart';

class ClosingStock<C extends ClosingStockCalibre>
    implements ClosingRangeAbstract {
  @override
  final ClosingRange closingRange;
  final List<C> articleCalibres;

  const ClosingStock(
    this.closingRange, {
    required this.articleCalibres,
  });

  static final dummy = ClosingStock(ClosingRange.dummyFeb, articleCalibres: [
    ClosingStockCalibre(
      ClosingRange.dummyFeb.date,
      id: 1,
      title: 'fromage abondance kg',
      initialQtCl: 0.0,
      finalQtCl: 5.0,
      articles: [
        ClosingStockArticle(
          ClosingRange.dummyFeb.date,
          quantityIn: 0.0,
          quantityOut: 0.0,
          id: 1,
          calibreId: 1,
          designation: 'fromage abondance kg',
          unitsInOnePiece: 1.0,
          finalQtCl: 5.0,
          initialQtCl: 0.0,
          price: 0.0,
          cost: 0.0,
        ),
      ],
    ),
  ]);

  @override
  bool operator ==(Object other) => identical(closingRange, other);
  @override
  int get hashCode => closingRange.hashCode;

  Map<String, dynamic> toMap() {
    return {
      'closingRange': closingRange.toMap(),
      'articleCalibres': articleCalibres.map((x) => x.toMap()).toList(),
    };
  }

  factory ClosingStock.fromMap(Map<String, dynamic> map) {
    return ClosingStock(
      ClosingRange.fromMap(map['closingRange']),
      articleCalibres: map['articleCalibres'] == null
          ? []
          : List<C>.from(map['articleCalibres']
              .map((x) => ClosingStockCalibre.fromMap(x))),
    );
  }

  String toJson() => json.encode(toMap());

  factory ClosingStock.fromJson(String source) =>
      ClosingStock.fromMap(json.decode(source));
}
