import 'dart:convert';

import 'package:models_weebi/utils.dart';
import 'package:models_weebi/src/closings/abstract/closing_range.dart';
import 'package:models_weebi/src/closings/abstract/stock_quantity.dart';
import 'package:models_weebi/models.dart' show ArticleRetailOnTicket;

class ClosingStockArticle extends ArticleRetailOnTicket
    implements ClosingDateAbstract, StockQuantityAbstract {
  @override
  DateTime closingDate;
  @override
  double initialQtCl;
  @override
  double finalQtCl;

  double quantityIn;
  double quantityOut;
  final int articleId;
  ClosingStockArticle(
    this.closingDate, {
    required super.calibreId,
    required super.id,
    required super.designation,
    required super.unitsInOnePiece,
    required this.initialQtCl,
    required this.finalQtCl,
    required this.quantityIn,
    required this.quantityOut,
    required super.price,
    required super.cost,
  }) : articleId = id;

  @override
  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'calibreId': calibreId,
      'closingDate': closingDate.toIso8601String(),
      'initialQtCl': initialQtCl,
      'finalQtCl': finalQtCl,
      'quantityIn': quantityIn,
      'quantityOut': quantityOut,
      'designation': designation,
      'unitsInOnePiece': unitsInOnePiece,
    };
  }

  factory ClosingStockArticle.fromMap(Map<String, dynamic> map) {
    return ClosingStockArticle(
      DateTime.tryParse(map['closingDate']) ?? DatesWeebi.defaultDate,
      id: map['id'] ?? map['articleId'] as int,
      calibreId: map['calibreId'] as int,
      // price: map['price'] ?? 0,
      // cost: map['cost'] ?? 0,
      designation: map['designation'] as String,
      unitsInOnePiece: map['unitsInOnePiece'] == null
          ? 1.0
          : (map['unitsInOnePiece'] as num).toDouble(),
      quantityIn: map['quantityIn'] == null
          ? 0.0
          : (map['quantityIn'] as num).toDouble(),
      quantityOut: map['quantityOut'] == null
          ? 0.0
          : (map['quantityOut'] as num).toDouble(),
      initialQtCl: map['initialQtCl'] == null
          ? 0.0
          : (map['initialQtCl'] as num).toDouble(),
      finalQtCl:
          map['finalQtCl'] == null ? 0.0 : (map['finalQtCl'] as num).toDouble(),
      price: map['price'] == null ? 0.0 : (map['price'] as num).toDouble(),
      cost: map['cost'] == null ? 0.0 : (map['cost'] as num).toDouble(),
    );
  }
  @override
  String toJson() => json.encode(toMap());
}
