import 'package:models_weebi/src/closings/abstract/closing_range.dart';
import 'package:models_weebi/src/closings/abstract/stock_quantity.dart';
import 'package:models_weebi/src/closings/closing_range.dart';
import 'package:models_weebi/src/closings/closing_stock_article.dart';

import 'package:models_weebi/abstracts.dart';
import 'package:models_weebi/utils.dart';

class ClosingStockCalibre extends CalibreAbstract<ClosingStockArticle>
    implements ClosingDateAbstract, StockQuantityAbstract {
  @override
  DateTime closingDate;
  @override
  double initialQtCl;
  @override
  double finalQtCl;
  ClosingStockCalibre(
    this.closingDate, {
    required super.id,
    required super.title,
    required this.initialQtCl,
    required this.finalQtCl,
    required super.articles,
    bool status = true,
  }) : super(
          status: true,
          creationDate: DateTime.now(),
        );

  static final cStockCalibreDummy = ClosingStockCalibre(
    ClosingRange.dummyFeb.date,
    id: 1,
    title: 'fromage abondance',
    initialQtCl: 0.0,
    finalQtCl: 5.0,
    articles: [
      ClosingStockArticle(
        ClosingRange.dummyFeb.date,
        quantityIn: 5.0,
        quantityOut: 0.0,
        id: 1,
        calibreId: 1,
        designation: 'fromage abondance kg',
        unitsInOnePiece: 1.0,
        finalQtCl: 5.0,
        initialQtCl: 0.0,
        price: 0.0,
        cost: 0.0,
      ),
    ],
  );

  @override
  Map<String, dynamic> toMap() {
    return {
      'closingDate': closingDate.toIso8601String(),
      'id': id,
      'title': title,
      'initialQtCl': initialQtCl,
      'finalQtCl': finalQtCl,
      'articles': articles.map((e) => e.toMap()).toList(),
    };
  }

  factory ClosingStockCalibre.fromMap(Map<String, dynamic> map) =>
      ClosingStockCalibre(
        DateTime.tryParse(map['closingDate']) ?? DatesWeebi.defaultDate,
        id: map['id'] as int,
        title: map['title'] as String,
        articles: map['articles'] == null
            ? []
            : List<ClosingStockArticle>.from(
                map['articles']?.map((x) => ClosingStockArticle.fromMap(x))),
        initialQtCl: map['initialQtCl'] == null
            ? 0.0
            : (map['initialQtCl'] as num).toDouble(),
        finalQtCl: map['finalQtCl'] == null
            ? 0.0
            : (map['finalQtCl'] as num).toDouble(),
      );

  @override
  bool operator ==(Object other) => identical(closingDate, other);
  @override
  int get hashCode => closingDate.hashCode;
}
