import 'dart:convert';
import 'package:models_weebi/src/closings/abstract/closing_contact.dart';
import 'package:models_weebi/src/closings/closing_range.dart';
import 'package:models_weebi/src/closings/closing_stock.dart';
import 'package:models_weebi/src/closings/closing_stock_article_calibre.dart';
import 'package:models_weebi/src/closings/closing_stock_article.dart';
import 'package:models_weebi/abstracts.dart' show CalibreAbstract;

import 'package:models_weebi/utils.dart' show DateRange;
import 'package:models_weebi/src/reports/report_stock_article_calibre.dart';

extension FinFlowClosingContactStock on List<ClosingStockContact> {
  List<ReportStockArtCalibre> herderReportStockArtCalibre(
    int herderId,
    List<CalibreAbstract> products,
    DateRange dateRange,
    List<ReportStockArtCalibre> emptyReportStockCalibres,
  ) {
    forEach((c) {
      if (c.contactId == herderId &&
          (c.closingRange.start.isAfter(dateRange.start) ||
              c.closingRange.start.isAtSameMomentAs(dateRange.start)) &&
          (c.closingRange.end.isBefore(dateRange.end) ||
              c.closingRange.end.isAtSameMomentAs(dateRange.end))) {
        for (var p in products) {
          if (c.articleCalibres.any((i) => i.id == p.id)) {
            // NOT any products can be both bought & sold
            // so we SUM ALL movements
            emptyReportStockCalibres
                    .firstWhere((reportStockP) => reportStockP.id == p.id)
                    .finalQtCl +=
                c.articleCalibres
                    .firstWhere((clsoingStockP) => clsoingStockP.id == p.id)
                    .finalQtCl;
          } else {
            print('closings do not fit requirements');
          }
        }
      }
    });
    return emptyReportStockCalibres;
  }
}

class ClosingStockContact extends ClosingStock
    implements ClosingContactAbstract {
  @override
  int contactId;
  ClosingStockContact(super.closingRange,
      {required this.contactId, List<ClosingStockCalibre>? articleCalibres})
      : super(articleCalibres: articleCalibres ?? []);

  @override
  Map<String, dynamic> toMap() {
    return {
      'closingRange': closingRange.toMap(), // ?
      'contactId': contactId,
      'articleCalibres': articleCalibres.map((e) => e.toMap()).toList(),
    };
  }

  factory ClosingStockContact.fromJson(String source) =>
      ClosingStockContact.fromMap(json.decode(source));

  factory ClosingStockContact.fromMap(Map<String, dynamic> map) {
    return ClosingStockContact(
      // ignore: missing_required_param
      ClosingRange.fromMap(map['closingRange']),
      articleCalibres: map['articleCalibres'] == null
          ? []
          : List<ClosingStockCalibre>.from(map['articleCalibres']
                  ?.map((x) => ClosingStockCalibre.fromMap(x)) ??
              const []),
      contactId: map['contactId'] ?? '',
    );
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ClosingStockContact && other.contactId == contactId;
  }

  @override
  int get hashCode => contactId.hashCode;

  static final milkDummy = [
    ClosingStockContact(ClosingRange.dummyLastMonth,
        contactId: 1,
        articleCalibres: [
          ClosingStockCalibre(
            ClosingRange.dummyLastMonth.date,
            id: 6,
            title: 'milk',
            initialQtCl: 0.0,
            finalQtCl: 20.0,
            articles: [
              ClosingStockArticle(
                ClosingRange.dummyLastMonth.date,
                quantityIn: 20.0,
                quantityOut: 0.0,
                id: 1,
                calibreId: 6,
                designation: 'milk',
                unitsInOnePiece: 1.0,
                finalQtCl: 20.0,
                initialQtCl: 0.0,
                price: 0.0,
                cost: 0.0,
              ),
            ],
          ),
        ])
  ];
}
