import 'package:models_weebi/utils.dart';
import 'package:models_weebi/models.dart';

extension FindDupsCalibre on List<Calibre> {
  TwoLists<Calibre> findDupsById({required List<Calibre> newList}) {
    final noDups = <Calibre>[];
    final dups = <Calibre>[];
    if (isEmpty) {
      noDups.addAll(newList);
      return TwoLists<Calibre>(dups: dups, noDups: noDups);
    }
    final oldListIds = map((e) => e.id);
    final newListIds = newList.map((e) => e.id);
    for (final newId in newListIds) {
      if (oldListIds.contains(newId)) {
        dups.add(newList.firstWhere((e) => e.id == newId));
      } else {
        noDups.add(newList.firstWhere((e) => e.id == newId));
      }
    }
    return TwoLists(noDups: noDups, dups: dups);
  }

  TwoLists<Calibre> findDupsByTitle({required List<Calibre> newList}) {
    final noDups = <Calibre>[];
    final dups = <Calibre>[];

    if (isEmpty) {
      noDups.addAll(newList);
      return TwoLists<Calibre>(dups: dups, noDups: noDups);
    }
    Map<int, Calibre> mapOldTitles = {};
    for (var p in this) {
      mapOldTitles[p.titleHash] = p;
    }
    for (var calibre in newList) {
      if (mapOldTitles.keys.contains(calibre.titleHash)) {
        // We make sure that we update the dup list with the old values missing from files
        // it is only relevant here because we built updating objects with dummy values
        final originalCalibre = mapOldTitles[calibre.titleHash]!;
        if (calibre.isBasket) {
          final article = (calibre.articles.first as ArticleBasket).copyWith(
            calibreId: originalCalibre.id,
            creationDate: originalCalibre.creationDate,
            updateDate: DateTime.now(),
            proxies: (originalCalibre.articles.first as ArticleBasket).proxies,
            discountAmountSalesOnly:
                (originalCalibre.articles.first as ArticleBasket)
                    .discountAmountSalesOnly,
            status: (originalCalibre.articles.first as ArticleBasket).status,
            statusUpdateDate: (originalCalibre.articles.first as ArticleBasket)
                .statusUpdateDate,
          );
          calibre = calibre.copyWith(
              id: mapOldTitles[calibre.titleHash]!.id,
              creationDate: mapOldTitles[calibre.titleHash]!.creationDate,
              articles: [article]);
          dups.add(calibre);
        } else {
          final updatedArticles = <ArticleRetail>[];
          for (var i = 0; i < calibre.articles.length; i++) {
            final article = (calibre.articles[i] as ArticleRetail).copyWith(
              calibreId: originalCalibre.id,
              creationDate:
                  (originalCalibre.articles[i] as ArticleRetail).creationDate,
              updateDate: DateTime.now(),
              unitsInOnePiece: (originalCalibre.articles[i] as ArticleRetail)
                  .unitsInOnePiece,
              articleCode:
                  (originalCalibre.articles[i] as ArticleRetail).articleCode,
              status: (originalCalibre.articles[i] as ArticleRetail).status,
              statusUpdateDate: (originalCalibre.articles[i] as ArticleRetail)
                  .statusUpdateDate,
            );
            updatedArticles.add(article);
          }
          calibre = calibre.copyWith(
              id: mapOldTitles[calibre.titleHash]!.id,
              creationDate: mapOldTitles[calibre.titleHash]!.creationDate,
              articles: updatedArticles);
          dups.add(calibre);
        }
      } else {
        noDups.add(calibre);
      }
    }
    return TwoLists<Calibre>(dups: dups, noDups: noDups);
  }
}
