import 'package:models_weebi/extensions.dart';
import 'package:models_weebi/models.dart';

extension ArticlesHandler on Iterable<Calibre> {
  Set<String> get articlesNamesClean {
    final biggyBring = <String>{};
    final strings = List<String>.of([]);
    for (final line in this) {
      for (final article in line.articles) {
        strings.add(article.designation.clean);
      }
    }
    return biggyBring;
  }
}

extension ArticlesFilter on Iterable<Calibre> {
  Iterable<Calibre> get all =>
      where((ac) => ac.title != '*').where((ac) => ac.id >= 0); // stupid legacy

  List<Calibre> searchByTitleOrId(String queryString) {
    final oneList = [...searchByTitle(queryString), ...searchById(queryString)];
    return List<Calibre>.of(Set<Calibre>.of(oneList));
  }

  List<Calibre> searchByIdOrArticleNames(String queryString) {
    final oneList = [...searchByTitle(queryString), ...searchById(queryString)];
    return List<Calibre>.of(Set<Calibre>.of(oneList));
  }

  Set<String> searchByArticleNames(String queryString) {
    final articlesNames = <String>{};
    final cautiousLegacyClean = all.where((p) => p.isNotQuickSpend);
    for (final articleNameClean in cautiousLegacyClean.articlesNamesClean) {
      if (articleNameClean.contains(queryString.clean)) {
        articlesNames.add(articleNameClean);
      }
    }
    return articlesNames;
  }

  List<Calibre> searchByTitle(String queryString) {
    return List<Calibre>.of(all
        .where((p) => p.isNotQuickSpend)
        .where((p) => p.title.clean.contains(queryString.clean))
        .toList());
  }

  List<Calibre> searchById(String queryString) {
    return List<Calibre>.of(all
        .where((p) => p.isNotQuickSpend)
        .where((p) => p.id.toString().contains(queryString.trim()))
        .toList());
  }

  List<Calibre> get notDeactivated => List<Calibre>.of(where((p) => p.status));

  List<Calibre> get palpables =>
      List<Calibre>.of(where((p) => p.isPalpable ?? true));
}
