import 'package:models_weebi/extensions.dart';
import 'package:models_weebi/models.dart' show ContactWeebi;

extension ContactsFinder<C extends ContactWeebi> on Iterable<C> {
  Iterable<C> get all => where((c) => c.id != 0);

  Set<int> findContactsIdsWithNameOrTel(String queryString) {
    final idsFullName = findContactsWithFullName(queryString);
    final idsFirstName = findContactsWithFirstName(queryString);
    final idsLastName = findContactsWithLastName(queryString);
    final idsTel = findContactsWithTel(queryString);

    final fullSetOfIds = Set.of(<int>{});
    for (final id in idsFullName) {
      fullSetOfIds.add(id);
    }
    for (final id in idsFirstName) {
      fullSetOfIds.add(id);
    }
    for (final id in idsLastName) {
      fullSetOfIds.add(id);
    }
    for (final id in idsTel) {
      fullSetOfIds.add(id);
    }
    return fullSetOfIds;
  }

  Set<int> findContactsWithFullName(String queryString) {
    final herdersMatchIds = Set.of(<int>{});
    for (final e in concatFirstLastName.entries) {
      if (e.value.contains(queryString.clean)) {
        herdersMatchIds.add(e.key);
      }
    }
    return herdersMatchIds;
  }

  Set<int> findContactsWithFirstName(String queryString) {
    final herdersMatch = all
        .where((c) => c.firstName.clean.contains(queryString.clean))
        .toList();
    final herdersMatchIds = Set.of(<int>{});
    for (final herder in herdersMatch) {
      herdersMatchIds.add(herder.id);
    }
    return herdersMatchIds;
  }

  Set<int> findContactsWithLastName(String queryString) {
    final herdersMatch =
        all.where((c) => c.lastName.clean.contains(queryString.clean)).toList();
    final herdersMatchIds = Set.of(<int>{});
    for (final herder in herdersMatch) {
      herdersMatchIds.add(herder.id);
    }
    return herdersMatchIds;
  }

  Map<int, String> get concatFirstLastName {
    final herdersNames = <int, String>{};
    for (final herder in all) {
      final f = herder.firstName.clean;
      final l = herder.lastName.clean;
      herdersNames[herder.id] = '$f $l';
    }
    return herdersNames;
  }

  Set<int> findContactsWithTel(String queryString) {
    if (queryString[0] == '0') {
      queryString = queryString.substring(1);
    }
    final herdersMatch = all
        .where((c) => c.phone.toString().trim().contains(queryString.trim()))
        .toList();
    final herdersMatchIds = Set.of(<int>{});
    for (final herder in herdersMatch) {
      herdersMatchIds.add(herder.id);
    }
    return herdersMatchIds;
  }

  Set<C> idsToContacts(Set<int> ids) {
    final contacts = Set.of(<C>{});
    for (final herder in this) {
      if (ids.contains(herder.id)) {
        contacts.add(herder);
      }
    }
    return contacts;
  }
}
