import 'package:models_weebi/src/util/date_range.dart';

extension IsInRange on DateTime {
  bool isDateInRange(DateTime startDateRange, DateTime endDateRange) =>
      (isAfter(startDateRange) || isAtSameMomentAs(startDateRange)) &&
      (isBefore(endDateRange) || isAtSameMomentAs(endDateRange));

  bool isDateInDateRange(DateRange dateRange) =>
      (isAfter(dateRange.start) || isAtSameMomentAs(dateRange.start)) &&
      (isBefore(dateRange.end) || isAtSameMomentAs(dateRange.end));
}

extension DateTimeCopy on DateTime {
  DateTime copyWith({
    int? year,
    int? month,
    int? day,
    int? hour,
    int? minute,
    int? second,
    int? millisecond,
    int? microsecond,
  }) {
    return DateTime(
      year ?? this.year,
      month ?? this.month,
      day ?? this.day,
      hour ?? this.hour,
      minute ?? this.minute,
      second ?? this.second,
      millisecond ?? this.millisecond,
      microsecond ?? this.microsecond,
    );
  }
}

extension Year on DateTime {
  DateTime get thisYearStart => DateTime(year, 1, 1, 0, 0, 0, 0, 000001);

  DateTime get thisYearEnd => DateTime(year, 12, 31, 23, 59, 59, 999, 999);
  List<DateRange> get lastThreeYears {
    final yearStart = thisYearStart;
    final yearEnd = thisYearEnd;
    final thisYear = DateRange(yearStart, yearEnd);

    final previousYear = thisYear.copyWith(
      start: yearStart.copyWith(year: yearStart.year - 1),
      end: yearEnd.copyWith(year: yearEnd.year - 1),
    );
    final twoYearsAgo = previousYear.copyWith(
      start: previousYear.start.copyWith(year: previousYear.start.year - 1),
      end: previousYear.end.copyWith(year: previousYear.end.year - 1),
    );

    return [thisYear, previousYear, twoYearsAgo];
  }
}

extension Month on DateTime {
  DateTime get thisMonthStart => DateTime(year, month, 01, 0, 0, 0, 0, 000001);

  DateTime get thisMonthEnd => DateTime(
      year, month, DateTime(year, month + 1, 0).day, 23, 59, 59, 999, 999);

  DateTime get previousMonthLastDay => DateTime(
      year, month - 1, DateTime(year, month, 0).day, 23, 59, 59, 999, 999);

  DateTime get previousMonthFirstDay =>
      DateTime(year, month - 1, 01, 0, 0, 0, 0, 000001);

  List<DateRange> get lastThreeMonths {
    final monthStart = thisMonthStart;
    final monthEnd = thisDayLastSecond;
    final thisMonth = DateRange(monthStart, monthEnd);
    final oneMonthAgo = DateRange(
        monthStart.previousMonthFirstDay, monthEnd.previousMonthLastDay);
    final twoMonthsAgo = DateRange(oneMonthAgo.start.previousMonthFirstDay,
        oneMonthAgo.end.previousMonthLastDay);

    return [
      thisMonth,
      oneMonthAgo,
      twoMonthsAgo,
    ];
  }

  List<DateRange> get lastThirteenMonths {
    final monthStart = thisMonthStart;
    final monthEnd = thisDayLastSecond;
    final thisMonth = DateRange(monthStart, monthEnd);
    final oneMonthAgo = DateRange(
        monthStart.previousMonthFirstDay, monthEnd.previousMonthLastDay);
    final twoMonthsAgo = DateRange(oneMonthAgo.start.previousMonthFirstDay,
        oneMonthAgo.end.previousMonthLastDay);
    final threeMonthsAgo = DateRange(twoMonthsAgo.start.previousMonthFirstDay,
        twoMonthsAgo.end.previousMonthLastDay);
    final fourMonthsAgo = DateRange(threeMonthsAgo.start.previousMonthFirstDay,
        threeMonthsAgo.end.previousMonthLastDay);
    final fiveMonthsAgo = DateRange(fourMonthsAgo.start.previousMonthFirstDay,
        fourMonthsAgo.end.previousMonthLastDay);
    final sixMonthsAgo = DateRange(fiveMonthsAgo.start.previousMonthFirstDay,
        fiveMonthsAgo.end.previousMonthLastDay);
    final sevenMonthsAgo = DateRange(sixMonthsAgo.start.previousMonthFirstDay,
        sixMonthsAgo.end.previousMonthLastDay);
    final eightMonthsAgo = DateRange(sevenMonthsAgo.start.previousMonthFirstDay,
        sevenMonthsAgo.end.previousMonthLastDay);
    final nineMonthsAgo = DateRange(eightMonthsAgo.start.previousMonthFirstDay,
        eightMonthsAgo.end.previousMonthLastDay);
    final tenMonthsAgo = DateRange(nineMonthsAgo.start.previousMonthFirstDay,
        nineMonthsAgo.end.previousMonthLastDay);
    final elevenMonthsAgo = DateRange(tenMonthsAgo.start.previousMonthFirstDay,
        tenMonthsAgo.end.previousMonthLastDay);
    final twelveMonthsAgo = DateRange(
        elevenMonthsAgo.start.previousMonthFirstDay,
        elevenMonthsAgo.end.previousMonthLastDay);
    return [
      thisMonth,
      oneMonthAgo,
      twoMonthsAgo,
      threeMonthsAgo,
      fourMonthsAgo,
      fiveMonthsAgo,
      sixMonthsAgo,
      sevenMonthsAgo,
      eightMonthsAgo,
      nineMonthsAgo,
      tenMonthsAgo,
      elevenMonthsAgo,
      twelveMonthsAgo,
    ];
  }
}

extension Week on DateTime {
  List<DateRange> get lastFourWeeks {
    var mondayThisWeek = this;
    while (mondayThisWeek.weekday != DateTime.monday) {
      mondayThisWeek = mondayThisWeek.subtract(const Duration(days: 1));
    }
    final monday = mondayThisWeek.thisDayFirstSecond;
    final sunday19Nov2023 = DateTime(
            mondayThisWeek.year, mondayThisWeek.month, mondayThisWeek.day + 6)
        .thisDayLastSecond;
    final thisWeek = DateRange(monday, sunday19Nov2023);
    final lastWeek = DateRange(monday.subtract(Duration(days: 7)),
        sunday19Nov2023.subtract(Duration(days: 7)));
    final twoWeeksAgo = DateRange(monday.subtract(Duration(days: 14)),
        sunday19Nov2023.subtract(Duration(days: 14)));
    final threeWeeksAgo = DateRange(monday.subtract(Duration(days: 21)),
        sunday19Nov2023.subtract(Duration(days: 21)));
    return [thisWeek, lastWeek, twoWeeksAgo, threeWeeksAgo];
  }
}

extension Day on DateTime {
  DateTime get thisDayFirstSecond =>
      DateTime(year, month, day, 0, 0, 0, 0, 000001);

  DateTime get thisDayLastSecond =>
      DateTime(year, month, day, 23, 59, 59, 999, 999);

  List<DateRange> get lastSevenDays {
    final start = thisDayFirstSecond;
    final end = thisDayLastSecond;
    final today = DateRange(thisDayFirstSecond, thisDayLastSecond);
    final yesterday = DateRange(
        start.subtract(Duration(days: 1)), end.subtract(Duration(days: 1)));
    final twoDaysAgo = DateRange(
        start.subtract(Duration(days: 2)), end.subtract(Duration(days: 2)));
    final threeDaysAgo = DateRange(
        start.subtract(Duration(days: 3)), end.subtract(Duration(days: 3)));
    final fourDaysAgo = DateRange(
        start.subtract(Duration(days: 4)), end.subtract(Duration(days: 4)));
    final fiveDaysAgo = DateRange(
        start.subtract(Duration(days: 5)), end.subtract(Duration(days: 5)));
    final sixDaysAgo = DateRange(
        start.subtract(Duration(days: 6)), end.subtract(Duration(days: 6)));

    return [
      today,
      yesterday,
      twoDaysAgo,
      threeDaysAgo,
      fourDaysAgo,
      fiveDaysAgo,
      sixDaysAgo
    ];
  }
}
