import 'package:models_weebi/common.dart'
    show Phone, PhotoSource, Address, Country;
import 'package:models_weebi/models.dart'
    show Calibre, ArticlePhoto, ArticleRetail, ContactWeebi, StockUnit;

extension ImportData on List<List<dynamic>> {
  /// bulk imports cannot deduce country dial code,
  /// avoid adding another column or a dodgy logic, nobody cares
  /// similiarly they cannot deduce check country code solidly enough
  List<ContactWeebi> extractContactsFromParsedCsvExcel(int nextContactId) {
    final contactsList = <ContactWeebi>[];
    var nextId = nextContactId; // just in case..

    for (final row in this) {
      if (row[0] != null && row[0].toString().trim() == 'firstName') {
        continue;
      }

      final street = row[4] != null ? row[4].toString().trim() : '';
      final code = row[5] != null ? row[5].toString().trim() : '';
      final city = row[6] != null ? row[6].toString().trim() : '';

      final address = (street.isNotEmpty || city.isNotEmpty || code.isNotEmpty)
          ? Address(
              street: street, city: city, code: code, country: Country.empty)
          : Address.empty;

      final now = DateTime.now();
      final herder = ContactWeebi(
        id: nextId,
        updateDate: now,
        status: true,
        statusUpdateDate: now,
        creationDate: DateTime.now(),
        firstName: row[0] != null ? row[0].toString().trim() : '',
        lastName: row[1] != null ? row[1].toString().trim() : '',
        phone: Phone(0, row[2] == null ? '' : row[2].toString().trim()),
        mail: row[3] != null ? row[3].toString().trim() : '',
        addressFull: address,
        categories:
            row[7] != null ? [row[7].toString().trim()] : const <String>[],
      );
      contactsList.add(herder);
      nextId++;
    }
    // print('contactsList.length ${contactsList.length}');
    return contactsList;
  }

  List<ArticlePhoto> get extractArticlesPhotosFromParsedExcel {
    final photosList = <ArticlePhoto>[];
    for (var i = 0; i < length; i++) {
      final table = this[i];
      final newPhoto = ArticlePhoto(
        calibreId: table[0] != null
            ? table[0].runtimeType == int
                ? table[0]
                : int.tryParse(table[1])
            : 0,
        id: table[1] != null
            ? table[1].runtimeType == int
                ? table[1]
                : int.tryParse(table[1])
            : 0,
        path: table[3] != null ? table[3].toString().trim() : '',
        source: PhotoSource.file,
      );
      photosList.add(newPhoto);
    }
    return photosList;
  }

  List<Calibre<ArticleRetail>> extractArticlesCalibresFromParsedCsvExcel(
      int nextLineId) {
    final linesList = <Calibre<ArticleRetail>>[];
    var nextId = nextLineId; // just in case..
    for (var i = 0; i < length; i++) {
      final now = DateTime.now();
      final table = this[i];
      final newArticle = ArticleRetail(
        calibreId: nextId,
        id: 1,
        articleCode: int.tryParse('${nextId}1'),
        creationDate: now,
        designation: table[0] != null ? table[0].toString().trim() : '',
        price: table[1] != null
            ? table[1].runtimeType == num
                ? table[1]
                : num.tryParse(table[1])
            : 0,
        cost: table[2] != null
            ? table[2].runtimeType == num
                ? table[2]
                : num.tryParse(table[2])
            : 0,
        unitsInOnePiece: 1.0,
        barcodeEAN: table[3] != null ? '${table[3]}' : '',
      );
      final newCalibre = Calibre<ArticleRetail>(
        creationDate: now,
        updateDate: now,
        id: nextId,
        title: table[0] != null ? table[0].toString().trim() : '',
        status: true,
        statusUpdateDate: DateTime.now(),
        stockUnit: StockUnit.unit,
        categories: [''],
        articles: [newArticle],
      );
      linesList.add(newCalibre);
      nextId++;
    }
    return linesList;
  }

  // * TOconsider factorize in one function with a boolean isUpdate
  List<Calibre<ArticleRetail>>
      extractCalibresOfArticlesForUpdateFromParsedCsvExcel() {
    final linesList = <Calibre<ArticleRetail>>[];
    for (var i = 0; i < length; i++) // avoid header
    {
      final now = DateTime.now();
      final table = this[i];
      final newArticle = ArticleRetail(
        creationDate: now, // will be updated after if match
        calibreId: 0, // will be updated after if match
        id: 1,
        designation: table[0] != null ? '${table[0]}' : '',
        price: table[1] != null
            ? table[1].runtimeType == num
                ? table[1]
                : num.tryParse(table[1])
            : 0,
        cost: table[2] != null
            ? table[2].runtimeType == num
                ? table[2]
                : num.tryParse(table[2])
            : 0,
        unitsInOnePiece: 1.0,
        barcodeEAN: table[3] != null ? '${table[3]}' : '',
      );
      final newLine = Calibre<ArticleRetail>(
        creationDate: now,
        updateDate: now,
        id: 0,
        title: table[0] != null ? '${table[0]}' : '',
        status: true,
        statusUpdateDate: DateTime.now(),
        stockUnit: StockUnit.unit,
        categories: [''],
        articles: [newArticle],
      );
      linesList.add(newLine);
    }
    return linesList;
  }

  /// bulk imports cannot deduce country dial code,
  /// avoid adding another column or a dodgy logic, nobody cares
  /// similiarly they cannot deduce check country code solidly enough
  List<ContactWeebi> extractContactsForUpdateFromParsedCsvExcel() {
    final contactsList = <ContactWeebi>[];
    for (var i = 0; i < length; i++) {
      final row = this[i];
      final street = row[4] != null ? row[4].toString().trim() : '';
      final code = row[5] != null ? row[5].toString().trim() : '';
      final city = row[6] != null ? row[6].toString().trim() : '';

      final address = (street.isNotEmpty || city.isNotEmpty || code.isNotEmpty)
          ? Address(
              street: street, city: city, code: code, country: Country.empty)
          : Address.empty;

      final now = DateTime.now();
      final herder = ContactWeebi(
        id: 0, // will be updated after if match
        creationDate: now, // will be updated after if match
        updateDate: now,
        status: true,
        statusUpdateDate: now,
        firstName: row[0] != null ? row[0].toString().trim() : '',
        lastName: row[1] != null ? row[1].toString().trim() : '',
        phone: Phone(0, row[2] == null ? '' : row[2].toString().trim()),
        mail: row[3] != null ? row[3].toString().trim() : '',
        addressFull: address,
        categories:
            row[7] != null ? [row[7].toString().trim()] : const <String>[],
      );
      contactsList.add(herder);
    }
    return contactsList;
  }
}
