import 'dart:convert';

import 'package:models_weebi/src/models/article/abstract/article_kind.dart';

abstract class ArticleRawAbstract {
  final int calibreId, id;
  final String designation;
  final ArticleKind kind;
  final bool status;
  const ArticleRawAbstract({
    required this.calibreId,
    required this.id,
    required this.designation,
    this.kind = ArticleKind.unknown,
    this.status = true,
  });

  Map<String, dynamic> toMap() {
    return {
      'calibreId': calibreId,
      'id': id,
      'designation': designation,
      'kind': kind,
    };
  }

  String toJson() => json.encode(toMap());

  String nameWithoutArtCalibrePart(String calibreTitle) {
    String name;
    if (calibreTitle.length < designation.length &&
        designation.contains(calibreTitle)) {
      name = designation.substring(calibreTitle.length);
    } else {
      name = designation;
    }
    return name.trim();
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ArticleRawAbstract &&
        other.id == id &&
        other.calibreId == calibreId &&
        other.designation == designation;
  }

  @override
  int get hashCode => id.hashCode ^ designation.hashCode;

  @override
  String toString() => 'ArticleRawAbstract(id: $id, designation: $designation)';
}
