import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:mobx/mobx.dart';

import 'package:models_weebi/models.dart' show StockUnit;
import 'package:models_weebi/src/models/article/abstract/article_raw_abstract.dart';

abstract class CalibreAbstract<A extends ArticleRawAbstract> {
  final int id;
  final String title;
  final StockUnit stockUnit;
  final DateTime? statusUpdateDate;
  final List<A> articles;
  final List<String>? categories;
  final DateTime? creationDate;
  final DateTime? updateDate;

  @observable
  final bool status;
  bool get hasSingleArticle => articles.length <= 1;

  const CalibreAbstract({
    required this.id,
    required this.title,
    required this.articles,
    this.stockUnit = StockUnit.unit,
    this.categories,
    this.statusUpdateDate,
    required this.creationDate,
    required this.status,
  }) : updateDate = creationDate;

  // factory ArtCalibre.fromJson(Map<String, dynamic> json) =>
  //     _$ArtCalibreFromJson(json);

  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'title': title,
      'stockUnit': stockUnit.toString(),
      'status': status,
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'articles': articles.map((x) => x.toMap()).toList(),
      'creationDate':
          creationDate == null ? '' : creationDate?.toIso8601String(),
      'updateDate': updateDate == null ? '' : updateDate?.toIso8601String(),
      'categories': categories?.map((e) => e).toList(),
    };
  }

  String toJson() => json.encode(toMap());

  @override
  String toString() {
    return '''
Calibre(
  id: $id,
  title: $title,
  stockUnit: $stockUnit,
  statusUpdateDate: $statusUpdateDate,
  updateDate: $updateDate,
  creationDate: $creationDate,
  status: $status,
  articles: $articles)  
    ''';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is CalibreAbstract<A> &&
        other.id == id &&
        other.title == title &&
        other.stockUnit == stockUnit &&
        other.status == status &&
        other.statusUpdateDate == statusUpdateDate &&
        listEquals(other.articles, articles) &&
        other.creationDate == creationDate &&
        other.updateDate == updateDate;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        title.hashCode ^
        stockUnit.hashCode ^
        status.hashCode ^
        statusUpdateDate.hashCode ^
        articles.hashCode ^
        creationDate.hashCode ^
        updateDate.hashCode;
  }
}
