import 'dart:convert';

import 'package:mobx/mobx.dart';
import 'package:models_weebi/src/models/article/abstract/article_code.dart';
import 'package:models_weebi/src/models/article/abstract/article_kind.dart';
import 'package:models_weebi/src/code_related_abstract_classes/default_attributes.dart';
import 'package:models_weebi/src/models/article/abstract/article_raw_retail_abstract.dart';

import 'package:models_weebi/utils.dart';
import 'package:models_weebi/src/models/article/price_and_cost.dart';

class ArticleRetailOnTicket extends ArticleRetailRawAbstract {
  const ArticleRetailOnTicket({
    super.kind = ArticleKind.retail,
    required super.calibreId,
    required super.id,
    required super.price,
    required super.cost,
    super.status = true,
    required super.designation,
    required super.unitsInOnePiece,
  });

  static const dummyRetailOnTicket = ArticleRetailOnTicket(
      calibreId: 1,
      id: 1,
      kind: ArticleKind.retail,
      designation: 'dummy retail',
      unitsInOnePiece: 1,
      price: 100,
      cost: 80);

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'calibreId': calibreId,
      'id': id,
      'kind': kind.toString(),
      'designation': designation,
      'unitsInOnePiece': unitsInOnePiece,
      'price': price,
      'cost': cost,
    };
  }

  factory ArticleRetailOnTicket.fromMap(Map<String, dynamic> map) {
    return ArticleRetailOnTicket(
      calibreId: map['calibreId']?.toInt() ?? 0,
      id: map['id']?.toInt() ?? 0,
      kind: ArticleKind.retail,
      designation: map['designation'] ?? (map['fullName'] ?? ''),
      unitsInOnePiece: map['unitsInOnePiece'] != null
          ? map['unitsInOnePiece']?.toDouble() ?? 1.0
          : map['weight'] != null
              ? map['weight']?.toDouble() ?? 1.0
              : 1.0,
      price: Price(map['price'] as num).price,
      cost: Cost(map['cost'] as num).cost,
    );
  }

  factory ArticleRetailOnTicket.fromJson(String source) =>
      ArticleRetailOnTicket.fromMap(json.decode(source));

  @override
  String toString() {
    return 'ArticleOnTicket(calibreId: $calibreId, id: $id, designation: $designation, unitsInOnePiece: $unitsInOnePiece, price: $price, cost: $cost)';
  }
}

class ArticleRetail extends ArticleRetailRawAbstract
    implements Price, Cost, DefaultDates, ArticleCode {
  String barcodeEAN;
  int get codeShortcut => articleCode ?? id;

  @override
  final int? articleCode;

  @override
  final DateTime creationDate;
  @override
  final DateTime? updateDate;
  @override
  final DateTime? statusUpdateDate;
  ArticleRetail({
    required super.calibreId,
    required super.id,
    required super.designation,
    required super.price,
    super.kind = ArticleKind.retail,
    super.cost = 0,
    this.articleCode,
    required this.creationDate,
    this.updateDate,
    this.statusUpdateDate,
    @observable super.status = true,
    super.unitsInOnePiece = 1,
    this.barcodeEAN = '',
  });

  static final dummy = ArticleRetail(
    calibreId: 1,
    id: 1,
    designation: 'dummy',
    price: 100,
    cost: 80,
    unitsInOnePiece: 1,
    articleCode: 1,
    barcodeEAN: 'barcodeEAN',
    creationDate: DatesWeebi.defaultDate,
    updateDate: DatesWeebi.defaultDate,
    statusUpdateDate: DatesWeebi.defaultDate,
    status: true,
  );

  static final dummyDecimal = ArticleRetail(
    calibreId: 3,
    id: 1,
    designation: 'dummy',
    price: 9.99,
    cost: 7.4,
    unitsInOnePiece: 1,
    articleCode: 1,
    barcodeEAN: 'barcodeEAN',
    creationDate: DatesWeebi.defaultDate,
    status: true,
  );

  @override
  String toString() {
    return """
ArticleWeebi(
  calibreId: $calibreId,
  id: $id,
  designation: '$designation',
  price: $price,
  cost: $cost,
  unitsInOnePiece: $unitsInOnePiece,
  articleCode: $articleCode,
  creationDate: $creationDate,
  updateDate: $updateDate,
  statusUpdateDate: $statusUpdateDate,
  status: $status,
  barcodeEAN: $barcodeEAN,
)
""";
  }

  @override
  Map<String, dynamic> toMap() {
    return {
      'calibreId': calibreId,
      'id': id,
      'designation': designation,
      'price': price,
      'cost': cost,
      'unitsInOnePiece': unitsInOnePiece,
      'barcodeEAN': barcodeEAN,
      'articleCode': articleCode ?? 0,
      'creationDate': creationDate.toIso8601String(),
      'updateDate': updateDate?.toIso8601String(),
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'status': status,
    };
  }

  ArticleRetailOnTicket get lightOnTicket => ArticleRetailOnTicket(
        calibreId: calibreId,
        id: id,
        price: price,
        cost: cost,
        designation: designation,
        unitsInOnePiece: unitsInOnePiece,
      );

  factory ArticleRetail.fromMap(Map<String, dynamic> map) {
    return ArticleRetail(
      calibreId: map['calibreId'] != null
          ? map['calibreId'] as int
          : map['lineId'] != null
              ? map['lineId'] as int
              : map['productId'] as int,
      id: map['id'] as int,
      designation: map['designation'] == null
          ? map['fullName'] as String
          : map['designation'] as String,
      kind: ArticleKind.retail,
      price: Price(map['price'] as num).price,
      cost: Cost(map['cost'] as num).cost,
      unitsInOnePiece: map['unitsInOnePiece'] != null
          ? (map['unitsInOnePiece'] as num).toDouble()
          : map['weight'] != null
              ? (map['weight'] as num).toDouble()
              : 1.0,
      articleCode: map['articleCode'] ?? 0,
      barcodeEAN: (map['barcodeEAN'] ?? '') as String,
      creationDate: map['creationDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['creationDate']),
      updateDate: map['updateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['updateDate']),
      status: map['status'],
      statusUpdateDate: map['statusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['statusUpdateDate']),
    );
  }

  @override
  String toJson() => json.encode(toMap());

  factory ArticleRetail.fromJson(String source) =>
      ArticleRetail.fromMap(json.decode(source));

  ArticleRetail copyWith({
    int? calibreId,
    int? id,
    String? designation,
    num? price,
    num? cost,
    double? unitsInOnePiece,
    int? articleCode,
    String? barcodeEAN,
    DateTime? creationDate,
    DateTime? updateDate,
    DateTime? statusUpdateDate,
    bool? status,
  }) {
    return ArticleRetail(
      calibreId: calibreId ?? this.calibreId,
      id: id ?? this.id,
      designation: designation ?? this.designation,
      price: price ?? this.price,
      cost: cost ?? this.cost,
      unitsInOnePiece: unitsInOnePiece ?? this.unitsInOnePiece,
      articleCode: articleCode ?? this.articleCode,
      barcodeEAN: barcodeEAN ?? this.barcodeEAN,
      creationDate: creationDate ?? this.creationDate,
      updateDate: updateDate ?? this.updateDate,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      status: status ?? this.status,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ArticleRetail &&
        other.cost == cost &&
        other.price == price &&
        other.designation == designation &&
        other.id == id &&
        other.calibreId == calibreId &&
        other.barcodeEAN == barcodeEAN &&
        other.creationDate == creationDate &&
        other.updateDate == updateDate;
  }

  @override
  int get hashCode => id.hashCode ^ calibreId.hashCode ^ creationDate.hashCode;
}
