import 'dart:convert';

import 'package:mobx/mobx.dart';
import 'package:models_weebi/abstracts.dart' show CalibreAbstract;
import 'package:models_weebi/models.dart' show StockUnit, ArticleBasket, ArticleRetail;
import 'package:models_weebi/src/models/article/abstract/article_raw_abstract.dart';
import 'package:models_weebi/utils.dart';
import 'package:models_weebi/src/demo/confitures_demo.dart';
import 'package:models_weebi/src/extensions/string_no_accents.dart';

import 'package:collection/collection.dart';

// if (isBasket){ articles.length ==1 } !

class Calibre<A extends ArticleRawAbstract> extends CalibreAbstract<A> {
  final bool? isPalpable; // impalpable are only used for quickSpend,
  // usually quickSpends are for electricity/transport/credit stuff one cannot touch
  // it is also a pun since these articles yields no gain / no money perceived through them
  // finally easier to add this boolean rather than handle another set of Article class and to cast everywhere
  // * will create another class ArticleRaw or NoTag or NoPriceNoCost
  // to remove this field and deduce base on type
  bool isBasket;
  bool get isSingleArticle => articles.length <= 1;
  bool get isNotQuickSpend => (isPalpable ?? true);
  String get nameLine => title;
  int get titleHash => title.withoutAccents.toLowerCase().trim().hashCode;

  Calibre({
    required super.id,
    this.isPalpable =
        true, // ? today this only mean quickspending i.e. with negative ids
    required super.articles,
    super.categories,
    required super.title,
    super.stockUnit,
    @observable super.status = true,
    super.statusUpdateDate,
    required super.creationDate,
    DateTime? updateDate,
  }) : isBasket = articles.isNotEmpty && articles.first is ArticleBasket;

// use a mixin ?
  String get sharableText {
    final truc =
        StringBuffer(); //Consider removing this inherited from old  old stock management

    final sb = StringBuffer()
      ..writeln('# $id - $title')
      ..writeln('stock : ${truc.toString()}');
    return sb.toString();
  }

  static final dummyRetail = Calibre<ArticleRetail>(
    id: 1,
    articles: [ArticleRetail.dummy],
    title: 'dummy',
    status: true,
    creationDate: DatesWeebi.defaultDate,
    updateDate: DatesWeebi.defaultDate,
    isPalpable: true,
  );

  static final dummyRetailDecimal = Calibre<ArticleRetail>(
    id: 3,
    articles: [ArticleRetail.dummyDecimal],
    title: 'dummyDecimal',
    status: true,
    creationDate: DatesWeebi.defaultDate,
    isPalpable: true,
  );

  static final dummyBasket = Calibre<ArticleBasket>(
      id: 2,
      categories: null,
      title: 'truc bis',
      creationDate: DatesWeebi.defaultDate,
      updateDate: DatesWeebi.defaultDate,
      stockUnit: StockUnit.unit,
      status: true,
      statusUpdateDate: DateTime.now(),
      articles: [ArticleBasket.dummy],
      isPalpable: true);

  @override
  Map<String, Object> toMap() {
    return {
      'id': id,
      'isPalpable': isPalpable ?? true,
      'title': title,
      'stockUnit': stockUnit.toString(),
      'status': status,
      'statusUpdateDate': statusUpdateDate?.toIso8601String() ??
          DatesWeebi.defaultDate.toIso8601String(),
      'articles': articles.map((x) => x.toMap()).toList(),
      'creationDate': creationDate?.toIso8601String() ??
          DatesWeebi.defaultDate.toIso8601String(),
      'updateDate': creationDate?.toIso8601String() ??
          DatesWeebi.defaultDate.toIso8601String(),
      'categories':
          categories == null ? <String>[] : categories!.map((e) => e).toList(),
    };
  }

  static Calibre<ArticleRetail> fromMapArticleRetail(Map<String, dynamic> map) {
    return Calibre<ArticleRetail>(
      id: map['id'],
      title: map['title'],
      isPalpable: map['isPalpable'] ?? true,
      stockUnit: StockUnit.tryParse(map['stockUnit'] ?? ''),
      creationDate: map['creationDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['creationDate']),
      updateDate: map['updateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['updateDate']),
      status: map['status'],
      statusUpdateDate: map['statusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['statusUpdateDate']),
      articles: map['articles'] == null || map['articles'] == []
          ? <ArticleRetail>[]
          : List<ArticleRetail>.from(
              map['articles'].map((x) => ArticleRetail.fromMap(x))),
      categories: map["categories"] == null
          ? []
          : List<String>.from(map["categories"].map((x) => x)),
    );
  }

  static Calibre<ArticleBasket> fromMapArticleBasket(Map<String, dynamic> map) {
    return Calibre<ArticleBasket>(
      id: map['id'],
      title: map['title'],
      isPalpable: map['isPalpable'] ?? true,
      stockUnit: StockUnit.tryParse(map['stockUnit'] ?? ''),
      creationDate: map['creationDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['creationDate']),
      updateDate: map['updateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['updateDate']),
      status: map['status'],
      statusUpdateDate: map['statusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['statusUpdateDate']),
      articles: map['articles'] == null || map['articles'] == []
          ? <ArticleBasket>[]
          : List<ArticleBasket>.from(
              map['articles'].map((x) => ArticleBasket.fromMap(x))),
      categories: map["categories"] == null
          ? []
          : List<String>.from(map["categories"].map((x) => x)),
    );
  }

  factory Calibre.fromJson(String source) =>
      Calibre.fromMap(json.decode(source));

  factory Calibre.fromMap(Map<String, dynamic> map) {
    return Calibre<A>(
      id: map['id'],
      title: map['title'],
      isPalpable: map['isPalpable'] ?? true,
      stockUnit: StockUnit.tryParse(map['stockUnit'] ?? ''),
      creationDate: map['creationDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['creationDate']),
      updateDate: map['updateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['updateDate']),
      status: map['status'],
      statusUpdateDate: map['statusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['statusUpdateDate']),
      articles: map['articles'] == null || map['articles'] == []
          ? []
          : List<A>.from(
              map['articles'].map((x) {
                if (x['discountAmount'] != null || x['proxies'] != null) {
                  return ArticleBasket.fromMap(x);
                } else {
                  return ArticleRetail.fromMap(x);
                }
              }),
            ),
      categories: map["categories"] == null
          ? []
          : List<String>.from(map["categories"].map((x) => x)),
    );
  }

  @override
  String toJson() => json.encode(toMap());

  Calibre<A> copyWith({
    int? id,
    String? title,
    bool? isPalpable,
    StockUnit? stockUnit,
    String? photo,
    bool? status,
    DateTime? statusUpdateDate,
    List<A>? articles,
    DateTime? creationDate,
    DateTime? updateDate,
    List<String>? categories,
  }) {
    return Calibre<A>(
      id: id ?? this.id,
      isPalpable: isPalpable ?? this.isPalpable,
      title: title ?? this.title,
      stockUnit: stockUnit ?? this.stockUnit,
      status: status ?? this.status,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      articles: articles ??
          this.articles.map((e) => e).toList(), // a real copy, not a reference
      creationDate: creationDate ?? this.creationDate,
      updateDate: updateDate ?? this.updateDate,
      categories: categories ?? this.categories,
    );
  }

  @override
  int get hashCode => id.hashCode ^ isPalpable.hashCode ^ creationDate.hashCode;

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    if (isBasket) {
      final listEquals = const DeepCollectionEquality().equals;
      return other is Calibre &&
          other.id == id &&
          other.isPalpable == isPalpable &&
          listEquals(other.articles as List<A>, articles);
    } else {
      return other is Calibre &&
          other.id == id &&
          other.isPalpable == isPalpable &&
          articles.first == articles.first;
    }
  }

  static final jams = CalibresDemoJamsBM.jamsData;
  static final jamsPhoto = CalibresDemoJamsBM.jamsPhotos;
}
