import 'dart:convert';

import 'package:mobx/mobx.dart';
import 'package:models_weebi/common.dart' show Address, Phone;

abstract class BoutiqueAbstract {
  final int id;
  final String managerMacAddress;
  final String name;
  final Phone phone;
  final String mail;
  final String shopKeeperName;
  final String shopKeeperTel;
  final String shopKeeperMail;
  final DateTime updateDate;
  @observable
  final bool status;
  final DateTime? statusUpdateDate;
  final bool serverStatus;
  final DateTime? serverStatusUpdateDate;
  final bool isProd;
  final bool isLocked;
  final double promo;

  final Address addressFull;

  BoutiqueAbstract({
    required this.id,
    required this.managerMacAddress,
    required this.name,
    required this.updateDate,
    required this.status,
    required this.statusUpdateDate,
    required this.serverStatus,
    required this.serverStatusUpdateDate,
    required this.isProd,
    required this.isLocked,
    required this.promo,
    this.phone = Phone.empty,
    this.mail = '',
    this.addressFull = Address.empty,
    this.shopKeeperName = '',
    this.shopKeeperTel = '',
    this.shopKeeperMail = '',
  });

  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'managerMacAddress': managerMacAddress,
      'name': name,
      'phone': phone.toMap(),
      'mail': mail,
      'addressFull': addressFull.toMap(),
      'shopKeeperName': shopKeeperName,
      'shopKeeperTel': shopKeeperTel,
      'shopKeeperMail': shopKeeperMail,
      'updateDate': updateDate.toIso8601String(),
      'status': status,
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'serverStatus': serverStatus,
      'serverStatusUpdateDate': serverStatusUpdateDate?.toIso8601String(),
      'isProd': isProd,
      'isLocked': isLocked,
      'promo': promo,
    };
  }

  String toJson() => json.encode(toMap());

  @override
  String toString() {
    return 'Boutique(id: $id, managerMacAddress: $managerMacAddress,  name: $name, phone: ${phone.toString()}, mail: $mail, addressFull: ${addressFull.toString()}, shopKeeperName: $shopKeeperName, shopKeeperTel: $shopKeeperTel, shopKeeperMail: $shopKeeperMail, updateDate: $updateDate, status: $status, statusUpdateDate: $statusUpdateDate, serverStatus: $serverStatus, serverStatusUpdateDate: $serverStatusUpdateDate, isProd: $isProd, isLocked: $isLocked, promo: $promo)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is BoutiqueAbstract &&
        other.id == id &&
        other.managerMacAddress == managerMacAddress &&
        other.name == name &&
        other.phone == phone &&
        other.mail == mail &&
        other.addressFull == addressFull &&
        other.shopKeeperName == shopKeeperName &&
        other.shopKeeperTel == shopKeeperTel &&
        other.shopKeeperMail == shopKeeperMail &&
        other.updateDate == updateDate &&
        other.status == status &&
        other.statusUpdateDate == statusUpdateDate &&
        other.serverStatus == serverStatus &&
        other.serverStatusUpdateDate == serverStatusUpdateDate &&
        other.isProd == isProd &&
        other.isLocked == isLocked &&
        other.promo == promo;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        managerMacAddress.hashCode ^
        name.hashCode ^
        phone.hashCode ^
        mail.hashCode ^
        addressFull.hashCode ^
        shopKeeperName.hashCode ^
        shopKeeperTel.hashCode ^
        shopKeeperMail.hashCode ^
        updateDate.hashCode ^
        status.hashCode ^
        statusUpdateDate.hashCode ^
        serverStatus.hashCode ^
        serverStatusUpdateDate.hashCode ^
        isProd.hashCode ^
        isLocked.hashCode ^
        promo.hashCode;
  }
}
