import 'dart:convert';

import 'package:models_weebi/abstracts.dart' show BoutiqueAbstract;
import 'package:models_weebi/common.dart' show Address, Country, Phone;
import 'package:models_weebi/utils.dart';

// consider adding
// * String appVersionBuildNumber
// * also add String emailResult
extension IsInRange on DateTime {
  bool isDateInRange(DateTime startDateRange, DateTime endDateRange) =>
      (isAfter(startDateRange) || isAtSameMomentAs(startDateRange)) &&
      (isBefore(endDateRange) || isAtSameMomentAs(endDateRange));
}

class BoutiqueWeebi extends BoutiqueAbstract {
  String mailUnique;
  final DateTime? promoStart;
  final DateTime? promoEnd;

  BoutiqueWeebi({
    required super.id,
    this.mailUnique = '',
    required super.name,
    required super.phone,
    super.status = true,
    required super.statusUpdateDate,
    required super.serverStatus,
    required super.serverStatusUpdateDate,
    required super.isProd,
    required super.isLocked,
    required super.updateDate,
    required super.addressFull,
    super.promo = 0.0,
    this.promoStart,
    this.promoEnd,
    super.mail,
    super.shopKeeperName,
    super.shopKeeperTel,
    super.shopKeeperMail,
  }) : super(managerMacAddress: '');

  String get sharableText {
    final sb = StringBuffer()
      ..writeln('boutique : $name')
      ..writeln('phone : $phone')
      ..writeln('mail : $mail')
      ..writeln('adress : $addressFull')
      ..writeln('');
    return sb.toString();
  }

  static final unknownBoutiqueWeebi = BoutiqueWeebi(
    id: 911,
    mailUnique: '',
    name: 'Boutique',
    phone: Phone.empty,
    mail: '',
    addressFull: Address.empty,
    shopKeeperName: '',
    shopKeeperTel: '',
    shopKeeperMail: '',
    updateDate: DatesWeebi.defaultDate,
    status: false,
    statusUpdateDate: DatesWeebi.defaultDate,
    serverStatus: false,
    serverStatusUpdateDate: DatesWeebi.defaultDate,
    isProd: false,
    isLocked: false,
    promo: 0.0,
  );

  static final dummy = BoutiqueWeebi(
    id: 1,
    mailUnique: '',
    name: 'Magasin par défaut',
    phone: Phone.empty,
    mail: '',
    addressFull: Address.empty,
    shopKeeperName: '',
    shopKeeperTel: '',
    shopKeeperMail: '',
    updateDate: DatesWeebi.defaultDate,
    status: false,
    statusUpdateDate: DatesWeebi.defaultDate,
    serverStatus: false,
    serverStatusUpdateDate: DatesWeebi.defaultDate,
    isProd: false,
    isLocked: false,
    promo: 0.0,
  );

  BoutiqueWeebi copyWith({
    int? id,
    String? managerMacAddress,
    Address? addressFull,
    String? uuid,
    String? mailUnique,
    String? name,
    Phone? phone,
    String? mail,
    String? shopKeeperName,
    String? shopKeeperTel,
    String? shopKeeperMail,
    DateTime? updateDate,
    bool? status,
    DateTime? statusUpdateDate,
    bool? serverStatus,
    DateTime? serverStatusUpdateDate,
    bool? isProd,
    bool? isLocked,
    double? promo,
    DateTime? promoStart,
    DateTime? promoEnd,
  }) {
    return BoutiqueWeebi(
      id: id ?? this.id,
      mailUnique: mailUnique ?? this.mailUnique,
      name: name ?? this.name,
      phone: phone ?? this.phone,
      mail: mail ?? this.mail,
      addressFull: addressFull ?? this.addressFull,
      shopKeeperName: shopKeeperName ?? this.shopKeeperName,
      shopKeeperTel: shopKeeperTel ?? this.shopKeeperTel,
      shopKeeperMail: shopKeeperMail ?? this.shopKeeperMail,
      updateDate: updateDate ?? this.updateDate,
      status: status ?? this.status,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      serverStatus: serverStatus ?? this.serverStatus,
      serverStatusUpdateDate:
          serverStatusUpdateDate ?? this.serverStatusUpdateDate,
      isProd: isProd ?? this.isProd,
      isLocked: isLocked ?? this.isLocked,
      promo: promo ?? this.promo,
      promoStart: promoStart ?? this.promoStart,
      promoEnd: promoEnd ?? this.promoEnd,
    );
  }

  @override
  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'name': name,
      'mail': mail,
      'mailUnique': mailUnique,
      'addressFull': addressFull.toMap(),
      'phone': phone.toMap(),
      'shopKeeperName': shopKeeperName,
      'shopKeeperTel': shopKeeperTel,
      'shopKeeperMail': shopKeeperMail,
      'updateDate': updateDate.toIso8601String(),
      'status': status,
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'serverStatus': serverStatus,
      'serverStatusUpdateDate': serverStatusUpdateDate?.toIso8601String(),
      'isProd': isProd,
      'isLocked': isLocked,
      'promo': promo,
      'promoStart': promoStart == null ? '' : promoStart!.toIso8601String(),
      'promoEnd': promoEnd == null ? '' : promoEnd!.toIso8601String(),
    };
  }

  factory BoutiqueWeebi.fromMap(Map<String, dynamic> map) {
    return BoutiqueWeebi(
      id: map['id'],
      mailUnique: map['mailUnique'],
      name: map['name'],
      phone: map['phone'] != null
          ? Phone.fromMap(map['phone'])
          : map['phone'] == null && map['tel'] != null
              ? Phone(0, map['tel'])
              : Phone(0, ''),
      mail: map['mail'],
      addressFull: map['addressFull'] != null
          ? Address.fromMap(map['addressFull'])
          : map['address'] != null
              ? Address(
                  street: map['address'], city: '', country: Country.empty)
              : Address.empty,
      shopKeeperName: map['shopKeeperName'],
      shopKeeperTel: map['shopKeeperTel'],
      shopKeeperMail: map['shopKeeperMail'],
      updateDate: map['updateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['updateDate']),
      status: map['status'],
      statusUpdateDate: map['statusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['statusUpdateDate']),
      serverStatus: map['serverStatus'],
      serverStatusUpdateDate: map['serverStatusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['serverStatusUpdateDate']),
      isProd: map['isProd'],
      isLocked: map['isLocked'],
      promo: map['promo'] == null ? 0.0 : (map['promo'] as num).toDouble(),
      promoStart: map['promoStart'] == null ||
              DateTime.tryParse(map['promoStart']) == null
          ? DatesWeebi.defaultDate
          : DateTime.parse(map['promoStart']),
      promoEnd:
          map['promoEnd'] == null || DateTime.tryParse(map['promoEnd']) == null
              ? DatesWeebi.defaultDate
              : DateTime.parse(map['promoEnd']),
    );
  }
  @override
  String toJson() => json.encode(toMap());

  factory BoutiqueWeebi.fromJson(String source) =>
      BoutiqueWeebi.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Boutique(id: $id, mailUnique: $mailUnique, name: $name, phone: ${phone.toString()}, mail: $mail, addressFull: ${addressFull.toString()}, shopKeeperName: $shopKeeperName, shopKeeperTel: $shopKeeperTel, shopKeeperMail: $shopKeeperMail, updateDate: $updateDate, status: $status, statusUpdateDate: $statusUpdateDate, serverStatus: $serverStatus, serverStatusUpdateDate: $serverStatusUpdateDate, isProd: $isProd, isLocked: $isLocked, promo: $promo)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is BoutiqueWeebi && other.id == id;
  }

  @override
  int get hashCode => id.hashCode;
}
