// Dart imports:
import 'dart:convert';

class Phone {
  //ISO 3166 alpha-2
  final int countryCode;
  final String number;

  const Phone(
    this.countryCode,
    this.number,
  );

  static const empty = Phone(0, '');

  Phone copyWith({int? countryCode, String? number}) {
    return Phone(
      countryCode ?? this.countryCode,
      number ?? this.number,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'countryCode': countryCode,
      'number': number,
    };
  }

  factory Phone.fromMap(Map<String, dynamic> map) {
    return Phone(
      map['countryCode']?.toInt() ?? 0,
      map['number'] ?? '',
    );
  }

  String toJson() => json.encode(toMap());

  factory Phone.fromJson(String source) => Phone.fromMap(json.decode(source));

  @override
  String toString() {
    if (number.isEmpty) {
      return '';
    } else if (countryCode == 0) {
      return number;
    } else {
      return '+$countryCode $number';
    }
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Phone &&
        other.countryCode == countryCode &&
        other.number == number;
  }

  @override
  int get hashCode => countryCode.hashCode ^ number.hashCode;
}
