import 'dart:convert';
import 'package:models_weebi/src/extensions/string_no_accents.dart';
import 'package:models_weebi/src/models/common/phone.dart';
import 'package:models_weebi/src/models/contact/contact_abstract.dart';
import 'package:models_weebi/utils.dart';
import 'package:models_weebi/common.dart' show Address, Country;
import 'package:models_weebi/src/models/common/tristate.dart';

// TODO : add optins -
class ContactWeebi extends ContactAbstract {
  // TODO put this as an extension for l10n
  String get sharableText {
    final sbContact = StringBuffer();
    if (id == 0) {
      sbContact.writeln('');
    } else {
      sbContact.writeln('prenom : $firstName');
      sbContact.writeln('nom : $lastName');
      if (phone.toString().isNotEmpty) {
        sbContact.writeln('tel: ${phone.toString()}');
      }
      if (mail.isNotEmpty) {
        sbContact.writeln('mail: $mail');
      }
      if (addressFull != null &&
          addressFull?.city != null &&
          addressFull!.city.isNotEmpty) {
        sbContact.writeln('adresse: ${addressFull.toString()}');
      }
    }
    return sbContact.toString();
  }

  String get fullName => '$firstName $lastName';
  int get fullNameHash => (firstName.withoutAccents.toLowerCase().trim() +
          lastName.withoutAccents.toLowerCase().trim())
      .hashCode;
  int get mailHash => mail.withoutAccents.toLowerCase().trim().hashCode;
  int get telHash => phone.toString().toLowerCase().trim().hashCode;
  final Tristate isWomanTri; // new one to be introduced in March 2024
  final List<String> categories;
  final DateTime? creationDate;
  final Address? addressFull;

  ContactWeebi({
    required super.id,
    required super.firstName,
    required super.lastName,
    required this.creationDate,
    required super.updateDate,
    required super.statusUpdateDate,
    super.status = true,
    required this.addressFull,
    super.phone,
    super.mail,
    super.avatar,
    super.overdraft,
    this.isWomanTri = Tristate.unknown,
    this.categories = const <String>[],
  });

  static final defaultContact = ContactWeebi(
    id: 0,
    firstName: 'contact', // * don't change this
    lastName: 'inconnu', // * used for display
    addressFull: Address.empty,
    creationDate: DatesWeebi.defaultDate,
    updateDate: DatesWeebi.defaultDate,
    statusUpdateDate: DatesWeebi.defaultDate,
    status: true,
    isWomanTri: Tristate.unknown,
  );
  static final dummy = ContactWeebi(
    id: 1,
    firstName: 'John', // * don't change this
    lastName: 'Doe',
    mail: 'john@doe.com',
    phone: Phone.empty,
    addressFull: Address.empty,
    creationDate: DatesWeebi.defaultDate,
    updateDate: DatesWeebi.defaultDate,
    statusUpdateDate: DatesWeebi.defaultDate,
    status: true,
    isWomanTri: Tristate.unknown,
    avatar: 'avatar',
  );

  static final dummy2 = ContactWeebi(
    id: 2,
    firstName: 'Jimmy',
    lastName: 'Doe',
    phone: const Phone(33, '780086476'),
    mail: 'john@doe.com',
    addressFull: Address.empty,
    creationDate: DatesWeebi.defaultDate,
    updateDate: DatesWeebi.defaultDate,
    statusUpdateDate: DatesWeebi.defaultDate,
    status: true,
    isWomanTri: Tristate.unknown,
  );

  @override
  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'firstName': firstName,
      'lastName': lastName,
      'addressFull':
          addressFull == null ? Address.empty.toMap() : addressFull!.toMap(),
      'phone': phone.toMap(),
      'mail': mail,
      'avatar': avatar,
      'creationDate': creationDate?.toIso8601String() ?? DatesWeebi.defaultDate,
      'updateDate': updateDate?.toIso8601String(),
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'status': status,
      'overdraft': overdraft,
      'categories': categories,
      'isWomanTri': isWomanTri.toString(),
    };
  }

  @override
  String toJson() => json.encode(toMap());

  factory ContactWeebi.fromMap(Map<String, dynamic> map) {
    return ContactWeebi(
      id: map['id'],
      firstName: map['firstName'],
      lastName: map['lastName'],
      addressFull: map['addressFull'] != null
          ? Address.fromMap(map['addressFull'])
          : map['address'] != null
              ? Address(
                  street: map['address'], city: '', country: Country.empty)
              : Address.empty,
      phone: map['phone'] == null && map['tel'] != null
          ? Phone(0, map['tel'] as String)
          : Phone.fromMap(map['phone']),
      mail: map['mail'],
      avatar: map['avatar'],
      creationDate: map['creationDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.tryParse(map['creationDate']) == null
              ? DateTime.fromMillisecondsSinceEpoch(map['creationDate'])
              : DatesWeebi.defaultDate,
      updateDate: map['updateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.tryParse(map['updateDate']) == null
              ? DateTime.fromMillisecondsSinceEpoch(map['updateDate'])
              : DatesWeebi.defaultDate,
      statusUpdateDate: map['statusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.tryParse(map['statusUpdateDate']) == null
              ? DateTime.fromMillisecondsSinceEpoch(map['statusUpdateDate'])
              : DatesWeebi.defaultDate,
      status: map['status'],
      overdraft: map['overdraft'],
      categories: List<String>.from(map['categories'] ?? []),
      isWomanTri: map['isWomanTri'] == null
          ? Tristate.unknown
          : Tristate.tryParse(map['isWomanTri'] as String),
    );
  }
  factory ContactWeebi.fromJson(String source) =>
      ContactWeebi.fromMap(json.decode(source));

  ContactWeebi copyWith({
    int? id,
    String? firstName,
    String? lastName,
    Phone? phone,
    String? mail,
    Address? addressFull,
    DateTime? creationDate,
    DateTime? updateDate,
    DateTime? statusUpdateDate,
    bool? status,
    Tristate? isWomanTri,
    List<String>? categories,
    String? avatar,
    int? overdraft,
  }) {
    return ContactWeebi(
      id: id ?? this.id,
      firstName: firstName ?? this.firstName,
      lastName: lastName ?? this.lastName,
      addressFull: addressFull ?? this.addressFull,
      creationDate: creationDate ?? this.creationDate,
      updateDate: updateDate ?? this.updateDate,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      status: status ?? this.status,
      phone: phone ?? this.phone,
      mail: mail ?? this.mail,
      avatar: avatar ?? this.avatar,
      overdraft: overdraft ?? this.overdraft,
      categories: categories ?? this.categories,
      isWomanTri: isWomanTri ?? this.isWomanTri,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ContactWeebi &&
        other.id == id &&
        other.firstName == firstName &&
        other.lastName == lastName &&
        other.phone == phone &&
        other.mail == mail &&
        other.addressFull == addressFull;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        firstName.hashCode ^
        lastName.hashCode ^
        phone.hashCode ^
        mail.hashCode ^
        addressFull.hashCode;
  }
}
