import 'dart:convert';
import 'package:collection/collection.dart';
import 'package:models_weebi/src/models/article/abstract/article_raw_abstract.dart';

import 'package:models_weebi/src/models/article/abstract/basket_abstract.dart';
import 'package:models_weebi/src/models/article/basket_logic/proxy_article_worth.dart';

typedef ArticleCreator<A extends ArticleRawAbstract> = A Function();

abstract class ItemAbstract<A extends ArticleRawAbstract>
    extends ItemRawAbstract<A> implements BasketAbstract {
  @override
  final List<ProxyArticleWorth>? proxiesWorth;

  ArticleCreator<A> articleCreator;
  ItemAbstract(this.articleCreator, double quantity,
      {List<ProxyArticleWorth>? proxiesWorth})
      : proxiesWorth = proxiesWorth ?? [],
        super(articleCreator(), quantity);

  @override
  String toString() => "Item($article,$proxiesWorth,$quantity)";

  @override
  Map<String, dynamic> toMap() {
    return {
      'article': article.toMap(),
      'quantity': quantity,
      'proxies_worth': proxiesWorth?.map((x) => x.toMap()).toList(),
    };
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is ItemAbstract<A> &&
        other.article == article &&
        listEquals(other.proxiesWorth, proxiesWorth) &&
        other.quantity == quantity;
  }

  @override
  int get hashCode =>
      article.hashCode ^ proxiesWorth.hashCode ^ quantity.hashCode;
}

abstract class ItemRawAbstract<A extends ArticleRawAbstract> {
  final A article;
  double quantity;
  ItemRawAbstract(this.article, this.quantity);

  @override
  String toString() => "Item($article,$quantity)";

  Map<String, dynamic> toMap() =>
      {'article': article.toMap(), 'quantity': quantity};

  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ItemRawAbstract<A> &&
        other.article == article &&
        other.quantity == quantity;
  }

  @override
  int get hashCode => article.hashCode ^ quantity.hashCode;
}
