import 'package:models_weebi/abstracts.dart';
import 'package:models_weebi/models.dart';

// Using to have a clean interface

extension PriceCostGetter on TicketType {
  bool get isPrice =>
      this == TicketType.sell || this == TicketType.sellDeferred;
  bool get isCost =>
      this == TicketType.spend || this == TicketType.spendDeferred;
}

mixin TotalComputer {
  num promoVal(num itemsTotal, double promo) =>
      (itemsTotal * promo / 100).round();

  num totalTaxExcludedPromoIncluded(
          num itemsTotal, double promo, num discountAmount) =>
      itemsTotal - promoVal(itemsTotal, promo) - discountAmount;

  num taxesVal(
    num itemsTotal,
    double promo,
    num discountAmount,
    double taxePercentage,
  ) =>
      (taxePercentage > 0.0
              ? totalTaxExcludedPromoIncluded(
                      itemsTotal, promo, discountAmount) *
                  (taxePercentage / 100)
              : 0)
          .round();

  num totalTaxAndPromoIncluded(
    num itemsTotal,
    double promo,
    num discountAmount,
    double taxePercentage,
  ) =>
      totalTaxExcludedPromoIncluded(itemsTotal, promo, discountAmount) +
      taxesVal(itemsTotal, promo, discountAmount, taxePercentage);
}

abstract class TicketAbstract
    with TotalComputer
    implements TicketRawAbstract<ItemCartWeebi, TaxWeebi> {
  final String oid; // mongo _id
  final num discountAmount;
  @override
  final int contactId;

  @override
  final int id;

  @override
  final List<ItemCartWeebi> items;
  @override
  final TaxWeebi taxe;
  @override
  final double promo;
  @override
  final String comment;
  @override
  final num received;
  @override
  final DateTime date;
  @override
  final PaymentType paymentType;
  @override
  final TicketType ticketType;

  @override
  late bool status;
  @override
  final DateTime? statusUpdateDate;
  @override
  final DateTime creationDate;

  TicketAbstract({
    required this.id,
    required this.oid,
    required this.items,
    required this.taxe,
    required this.promo,
    required this.comment,
    required this.received,
    required this.date,
    required this.paymentType,
    required this.ticketType,
    required this.contactId,
    required this.status,
    required this.statusUpdateDate,
    required this.creationDate,
    this.discountAmount = 0,
  });

  // sell and sellDeferred
  num get totalPriceItemsOnly => ticketType.isPrice ? items.itemsTotalPrice : 0;

  num get totalPricePromoVal => promoVal(totalPriceItemsOnly, promo);

  num get totalPriceTaxExcludedPromoIncluded =>
      totalTaxExcludedPromoIncluded(totalPriceItemsOnly, promo, discountAmount);

  num get totalPriceTaxesVal =>
      taxesVal(totalPriceItemsOnly, promo, discountAmount, taxe.percentage);

  num get totalPriceTaxAndPromoIncluded => totalTaxAndPromoIncluded(
        totalPriceItemsOnly,
        promo,
        discountAmount,
        taxe.percentage,
      );
// spend and spendDeferred below
  num get totalCostItemsOnly => ticketType.isCost ? items.itemsTotalCost : 0;

  num get totalCostPromoVal => promoVal(totalCostItemsOnly, promo);

  num get totalCostTaxExcludedIncludingPromo =>
      totalTaxExcludedPromoIncluded(totalCostItemsOnly, promo, discountAmount);

  num get totalCostTaxesVal =>
      taxesVal(totalCostItemsOnly, promo, discountAmount, taxe.percentage);

  num get totalCostTaxAndPromoIncluded => totalTaxAndPromoIncluded(
      totalCostItemsOnly, promo, discountAmount, taxe.percentage);

  num get total {
    switch (ticketType) {
      case TicketType.sell:
        return totalPriceTaxAndPromoIncluded;
      case TicketType.sellDeferred:
        return totalPriceTaxAndPromoIncluded;
      case TicketType.sellCovered:
        return received;
      case TicketType.spend:
        return totalCostTaxAndPromoIncluded;
      case TicketType.spendDeferred:
        return totalCostTaxAndPromoIncluded;
      case TicketType.spendCovered:
        return received;
      case TicketType.wage:
        return received;
      case TicketType.unknown:
        print('unknow ticket type in calculateSumsOfTicket');
        return 0;
      default:
        return 0;
    }
  }

  String get deactivatedDate => status ? '' : '$statusUpdateDate';
}
