// ignore_for_file: unnecessary_null_comparison

import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:mobx/mobx.dart';

import 'package:models_weebi/src/models/item_in_cart/item_cart_abstract.dart';
import 'package:models_weebi/src/models/tax/tax_abstract.dart';
import 'package:models_weebi/src/models/ticket/logic/payment_type.dart';
import 'package:models_weebi/src/models/ticket/logic/ticket_type.dart';

abstract class TicketRawAbstract<I extends ItemAbstract,
    T extends TaxAbstract> {
  final int id;
  final List<I> items;
  final T taxe;
  final double promo;
  final String comment; // comment
  final num received;
  final DateTime date;
  final PaymentType paymentType;
  final TicketType ticketType;
  final int contactId;
  @observable
  final bool status;
  final DateTime? statusUpdateDate;
  final DateTime
      creationDate; // * using it as unique Key? to display the ticket

  const TicketRawAbstract({
    required this.id,
    required this.items,
    required this.taxe,
    this.promo = 0.0,
    this.comment = '',
    required this.received,
    required this.date,
    required this.contactId,
    required this.paymentType,
    required this.ticketType,
    required this.status,
    required this.statusUpdateDate,
    required this.creationDate,
  });

  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'items': items.map((x) => x.toMap()).toList(),
      'taxe': taxe.toMap(),
      'promo': promo,
      'comment': comment,
      'received': received,
      'date': date.toIso8601String(),
      'paymentType': paymentType.toString(),
      'ticketType': ticketType.toString(),
      'contactId': contactId,
      'status': status,
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'creationDate': creationDate.toIso8601String(),
    };
  }

  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is TicketRawAbstract<I, T> &&
        other.id == id &&
        listEquals(other.items, items) &&
        other.taxe == taxe &&
        other.promo == promo &&
        other.comment == comment &&
        other.received == received &&
        other.date == date &&
        other.paymentType == paymentType &&
        other.ticketType == ticketType &&
        other.contactId == contactId &&
        other.status == status &&
        other.statusUpdateDate == statusUpdateDate &&
        other.creationDate == creationDate;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        items.hashCode ^
        taxe.hashCode ^
        promo.hashCode ^
        comment.hashCode ^
        received.hashCode ^
        date.hashCode ^
        paymentType.hashCode ^
        ticketType.hashCode ^
        contactId.hashCode ^
        status.hashCode ^
        statusUpdateDate.hashCode ^
        creationDate.hashCode;
  }
}
