// ignore_for_file: unnecessary_null_comparison

import 'dart:convert';
import 'package:models_weebi/utils.dart';
import 'package:models_weebi/src/models/ticket/ticket_mixin_weebi_base.dart';
import 'package:collection/collection.dart';
import 'package:models_weebi/models.dart';

// getters are in the mixin => much lighter
// hence avoiding confusion with Item and ArticleBasket

class TicketWeebi extends TicketAbstract with TicketMixinWeebiBase {
  TicketWeebi({
    required super.oid, // mongo legacy
    required super.id,
    required super.items,
    required super.taxe,
    required super.promo,
    required super.comment,
    required super.received,
    required super.date,
    required super.paymentType,
    required super.ticketType,
    required super.contactId,
    super.status = true,
    required DateTime? statusUpdateDate,
    required super.creationDate,
    final num? discountAmount = 0,
  }) : super(
          statusUpdateDate: statusUpdateDate ?? DatesWeebi.defaultDate,
          discountAmount: discountAmount ?? 0,
        );

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is TicketWeebi &&
        other.id == id &&
        listEquals(other.items, items) &&
        other.taxe == taxe &&
        other.promo == promo &&
        other.comment == comment &&
        other.received == received &&
        other.date == date &&
        other.paymentType == paymentType &&
        other.ticketType == ticketType &&
        other.contactId == contactId &&
        other.status == status &&
        other.statusUpdateDate == statusUpdateDate &&
        other.discountAmount == discountAmount &&
        other.creationDate == creationDate;
  }

  @override
  String toString() => """
TicketWeebi{
      'id': $id,
      'items': ${items.map((x) => x.toMap()).toList()},
      'taxe': ${taxe.toMap()},
      'promo': $promo,
      'comment': $comment,
      'received': $received,
      'date': ${date.toIso8601String()},
      'paymentType': $paymentType,
      'ticketType': $ticketType,
      'contactId': $contactId,
      'status': $status,
      'statusUpdateDate': ${statusUpdateDate?.toIso8601String()},
      'creationDate': ${creationDate.toIso8601String()},
      'discountAmount': $discountAmount,
    };
  }
  """;

  @override
  int get hashCode {
    return id.hashCode ^
        items.hashCode ^
        taxe.hashCode ^
        promo.hashCode ^
        comment.hashCode ^
        received.hashCode ^
        date.hashCode ^
        paymentType.hashCode ^
        ticketType.hashCode ^
        contactId.hashCode ^
        statusUpdateDate.hashCode ^
        discountAmount.hashCode ^
        creationDate.hashCode;
  }

  static final dummySell = TicketWeebi(
    oid: 'oid',
    id: 1,
    items: [ItemCartWeebi.dummy],
    taxe: TaxWeebi.noTax,
    promo: 0.0,
    comment: 'comment',
    received: 0,
    date: DatesWeebi.defaultFirstDate,
    paymentType: PaymentType.cash,
    ticketType: TicketType.sell,
    contactId: 1,
    status: true,
    statusUpdateDate: DatesWeebi.defaultFirstDate,
    creationDate: DatesWeebi.defaultFirstDate,
    discountAmount: 0,
  );
  static final dummySpend = TicketWeebi(
    oid: 'oid',
    id: 2,
    items: [ItemCartWeebi.dummy],
    taxe: TaxWeebi.noTax,
    promo: 0.0,
    comment: 'comment',
    received: 0,
    date: DatesWeebi.defaultFirstDate,
    paymentType: PaymentType.cash,
    ticketType: TicketType.spend,
    contactId: 2,
    status: true,
    statusUpdateDate: DatesWeebi.defaultFirstDate,
    creationDate: DatesWeebi.defaultFirstDate,
    discountAmount: 0,
  );

  factory TicketWeebi.fromJson(String source) =>
      TicketWeebi.fromMap(json.decode(source));

  factory TicketWeebi.fromMap(Map<String, dynamic> map) {
    return TicketWeebi(
      id: map['id'] as int,
      oid: map['oid'] == null ? '' : map['oid'] as String,
      items: map['items'] == null
          ? <ItemCartWeebi>[]
          : List<ItemCartWeebi>.from(
              map['items'].map((x) => ItemCartWeebi.fromMap(x))),
      taxe: TaxWeebi.fromMap(map['taxe']),
      promo: map['promo'] == null ? 0.0 : (map['promo'] as num).toDouble(),
      discountAmount:
          map['discountAmount'] == null ? 0 : map['discountAmount'] as num,
      comment: map['comment'] ?? '',
      received: map['received'] as num,
      date: DateTime.tryParse(map['date']) ?? DatesWeebi.defaultDate,
      paymentType: map['paymentType'] == null
          // * old typo, do not change
          ? PaymentType.tryParse(map['paiementType'] as String)
          : PaymentType.tryParse(map['paymentType'] as String),
      ticketType: TicketType.tryParse(map['ticketType'] as String),
      contactId: (map['contactId'] != null)
          ? (map['contactId'] as int)
          // old typo do not change
          : int.tryParse(map['contactInfo']) ?? 0,
      status: map['status'] as bool,
      statusUpdateDate: map['statusUpdateDate'] == null
          ? DatesWeebi.defaultDate
          : DateTime.tryParse(map['statusUpdateDate']),
      creationDate:
          DateTime.tryParse(map['creationDate']) ?? DatesWeebi.defaultDate,
    );
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'oid': oid,
      'items': items.map((x) => x.toMap()).toList(),
      'taxe': taxe.toMap(),
      'promo': promo,
      'discountAmount': discountAmount,
      'comment': comment,
      'received': received,
      'date': date.toIso8601String(),
      'paymentType': paymentType.toString(),
      'ticketType': ticketType.toString(),
      'contactId': contactId,
      'status': status,
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'creationDate': creationDate.toIso8601String(),
    };
  }

  TicketWeebi copyWith({
    String? oid,
    int? id,
    int? shopId,
    List<ItemCartWeebi>? items,
    TaxWeebi? taxe,
    double? promo,
    num? discountAmount,
    String? comment,
    num? received,
    DateTime? date,
    PaymentType? paymentType,
    TicketType? ticketType,
    int? contactId,
    bool? status,
    DateTime? statusUpdateDate,
    DateTime? creationDate,
  }) {
    return TicketWeebi(
      oid: oid ?? this.oid,
      id: id ?? this.id,
      items: items ??
          this.items.map((e) => e).toList(), // a real copy, not a reference
      taxe: taxe ?? this.taxe,
      promo: promo ?? this.promo,
      comment: comment ?? this.comment,
      received: received ?? this.received,
      date: date ?? this.date,
      paymentType: paymentType ?? this.paymentType,
      ticketType: ticketType ?? this.ticketType,
      contactId: contactId ?? this.contactId,
      status: status ?? this.status,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      creationDate: creationDate ?? this.creationDate,
      discountAmount: discountAmount ?? this.discountAmount,
    );
  }

  String get getTicketTypeContactText {
    if (ticketType == TicketType.sell && contactId == 0) {
      return 'Client : Visiteur';
    } else if (ticketType == TicketType.sell && contactId != 0) {
      return 'Client id : $contactId';
    } else if (ticketType == TicketType.spend && contactId == 0) {
      return 'Fournisseur : Habituel';
    } else if (ticketType == TicketType.stockIn ||
        ticketType == TicketType.stockOut) {
      return '';
    } else {
      return 'id : $contactId';
    }
  }

  String get contactIdString => '$contactId';
}
