library report_financials;

import 'package:models_weebi/models.dart' show TicketType, FinFlowType;
import 'package:models_weebi/extensions.dart';
import 'package:models_weebi/src/closings/closing_ledger_contact.dart';
import 'package:models_weebi/src/closings/closing_ledger_boutique.dart';

import 'package:models_weebi/src/models/boutique/boutique_weebi.dart';
import 'package:models_weebi/src/models/ticket/ticket_weebi.dart';
import 'package:models_weebi/src/reports/compute_financials_mixin.dart';
import 'package:models_weebi/src/reports/report_abstracts.dart';
import 'package:models_weebi/utils.dart';
import 'package:models_weebi/models.dart' show ContactWeebi;

class ReportFinancialContact extends ReportFinancial<ClosingLedgerContact>
    with ComputeFinancialsContact {
  final ContactWeebi herder;
  ReportFinancialContact({
    required this.herder,
    required List<ClosingLedgerContact> closingContactLedgers,
    required super.tickets,
    required super.startDate,
    required super.endDate,
  }) : super(
          objectId: herder.id,
          closingsObjects: closingContactLedgers,
        );
}

extension AccountingCharts on Iterable<ReportFinancialBoutique> {
  double get cashFlowAverage =>
      (fold(0.0, (pv, e) => pv + e.financialFlows.cashflow) / length)
          .roundTwoDecimals;

  Map<TicketType, double> get totalPerType {
    final map = <TicketType, double>{};
    for (final report in this) {
      for (final ticketType in TicketType.financialTypes) {
        if (map[ticketType] != null) {
          map[ticketType] = ((map[ticketType])! +
              report.financialFlows.sumTotalForType(ticketType));
        } else {
          map[ticketType] = report.financialFlows.sumTotalForType(ticketType);
        }
      }
    }
    return map;
  }
}

extension PutReportFinancialInADatatable on Iterable<ReportFinancialBoutique> {
  Map<TicketType, Map<DateRange, double>> get cellsTotalMap {
    final temp = <CellTotal>[];
    for (final report in this) {
      for (final ticketType in TicketType.financialTypes) {
        temp.add(
          CellTotal(
            DateRange(report.start, report.end),
            ticketType,
            report.financialFlows.sumTotalForType(ticketType),
          ),
        );
      }
    }
    final rows = <TicketType, Map<DateRange, double>>{};
    for (final fin in TicketType.financialTypes) {
      rows[fin] = {};
    }
    for (final row in rows.entries) {
      final cells = <DateRange, double>{};
      for (final e in temp.where((element) => element.ticketType == row.key)) {
        cells[e.dateRange] = e.total;
      }
      row.value.addAll(cells);
    }

    return rows;
  }
}

class ReportFinancialBoutique extends ReportFinancial<ClosingLedgerBoutique>
    with ComputeFinancialsBoutique {
  final int shopId;
  ReportFinancialBoutique({
    required this.shopId,
    required List<ClosingLedgerBoutique> closingLedgerBoutiques,
    required super.tickets,
    required super.startDate,
    required super.endDate,
  }) : super(
          objectId: shopId,
          closingsObjects: closingLedgerBoutiques,
        );
  static final dummy = ReportFinancialBoutique(
      shopId: BoutiqueWeebi.dummy.id,
      tickets: [TicketWeebi.dummySell],
      closingLedgerBoutiques: [],
      startDate: DatesWeebi.defaultFirstDate.add(const Duration(days: 1)),
      endDate: DatesWeebi.defaultLastDate.subtract(const Duration(days: 1)));
}
