// import 'package:flutter/material.dart';
// DateTimeRange is incompatible with Dart
import 'package:models_weebi/src/util/weebi_dates.dart';

extension DateRangeExt on DateRange {
  Duration get duration => end.difference(start);

  DateRange copyWith({
    DateTime? start,
    DateTime? end,
  }) =>
      DateRange(
        start ?? this.start,
        end ?? this.end,
      );
}

class DateRange {
  final DateTime start;
  final DateTime end;
  DateRange(this.start, this.end) : assert(!start.isAfter(end));

  static final defaultDateRange =
      DateRange(DatesWeebi.defaultFirstDate, DatesWeebi.defaultLastDate);

  @override
  bool operator ==(Object other) {
    if (other.runtimeType != runtimeType) {
      return false;
    }
    return other is DateRange && other.start == start && other.end == end;
  }

  @override
  int get hashCode => Object.hash(start, end);

  @override
  String toString() =>
      ('${start.toIso8601String()} - ${end.toIso8601String()}');
}
