class Timespan {
  final String _range;
  const Timespan._(this._range);

  @override
  String toString() => _range;

  static const Timespan day = Timespan._('day');
  static const Timespan daySeven = Timespan._('day_seven');
  static const Timespan week = Timespan._('week');
  static const Timespan weekFour = Timespan._('week_four');
  static const Timespan month = Timespan._('month');
  static const Timespan monthThree = Timespan._('month_three');
  static const Timespan monthThirteen = Timespan._('month_thirteen');
  static const Timespan year = Timespan._('year');
  static const Timespan yearThree = Timespan._('year_three');
  static const Timespan custom = Timespan._('custom');
  static const Timespan full = Timespan._('full');
  static const Timespan unknown = Timespan._('');

  static Timespan tryParse(String val) {
    switch (val) {
      case 'day':
        return Timespan.day;
      case 'week':
        return Timespan.week;
      case 'month':
        return Timespan.month;
      case 'year':
        return Timespan.year;
      case 'custom':
        return Timespan.custom;
      case 'full':
        return Timespan.full;
      default:
        print('$val is not a valid Timespan');
        return Timespan.unknown;
    }
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Timespan && other._range == _range;
  }

  @override
  int get hashCode => _range.hashCode;

  int get chartLength {
    switch (this) {
      case Timespan.daySeven:
        return 7;
      case Timespan.weekFour:
        return 4;
      case Timespan.monthThree:
        return 3;
      case Timespan.monthThirteen:
        return 13;
      case Timespan.yearThree:
        return 3;
      default:
        throw 'unhandled timespan for charts length';
    }
  }

  String get properString {
    if (_range == daySeven.toString()) {
      return '7_jours';
    } else if (_range == weekFour.toString()) {
      return '4_semaines';
    } else if (_range == monthThree.toString()) {
      return '3_mois';
    } else if (_range == monthThirteen.toString()) {
      return '13_mois';
    } else if (_range == yearThree.toString()) {
      return '3_ans';
    } else {
      return '?';
    }
  }
}
